/*****************************************************
 * G-Tronic Robotics -  GT-Bot V1.1                  *
 * Versione 12/04/2015  Con display 4bit telemetrico *
 *  Settaggi del PIC                                 *
 *                                                   *
 *                                                   *
 ****************************************************/

void settaggi(){
	
    #define _LEGACY_HEADERS  // permette di acquisire la sintassi dei compilatori precedenti
	
	
    __CONFIG (HS & WDTEN & PWRTDIS & BORDIS & LVPDIS & DEBUGDIS & UNPROTECT); //Fuses
	/*
     * HS       -> settaggio oscillatore a 20Mhz, altre opzioni: XT, LP, RC
     * WDTEN    -> abilita il watch dog timer, WDTDIS lo disabilita
     * PWRTDIS  -> disabilita lo start up timer
     * BORDIS
     * LVPDIS   -> disabilita low voltage programming
     * DUNPROT  -> disabilita data code area protection
     * WRTEN    -> abilita la scrittura nella flasch area
     * UNPROTECT-> disable program code protection
     */
	
	
    //ADCON1 = 0b10000000; // abilita gli ingressi analogici AN0 .. AN7 su PORTA
    ADCON1 = 0b00000111; // disabilita gli ingressi analogici AN0 .. AN7 su PORTA
    ANSELH=0b00000000;   // disabilita comparatori
	
	
	// SETTAGGIO DEI PORT
    /*
     * setto le porte A B C D E come ingressi ( = 1 ) o come uscite ( = 0 )
     * regola importante, per evitare qualsiasi tipo di disturbo se non uso una
     * porta la setto come output
     * 
     * N.B. 
     * 
     * TRISX = 0 equivale a TRISX = 0b00000000
     * 
     * TRISX = 1 NON equivale a TRISX = 0b11111111 
     * ma equivale a TRISX = 0b00000001
     */
    TRISA = 0;  
    TRISB = 0xFF;  // interamente connesso a 8 led on board
    TRISC = 0b11100000; // TRISC = 0xC0 // per abilitare la comunicazione seriale
    TRISD = 0; 
    TRISE = 0; 
	
    PORTA = 0;
    PORTB = 0;
    PORTC = 0;
    PORTD = 0;
    PORTE = 0;
 
	// interrupt
    INTCON = 0b10000100;  // abilito GIE dell'INCON register
    /* registro INTCON
     * GIE general inerrupt enable
     * PIEIE interrupt generato da periferiche interne o esterne (COM - USB)
     * T0IE abilitazione interrupt da timer zero TMR0
     * INTE (sempre 1, lavora insieme a GIE) interrupt abilitato
     * RBIE abilita interrupt su cambio di stato del RB4,RB5,RB6,RB7
     * T0IF verifica presenza interrupt da TMR0
     * INTF verifica presenza interrupt dal PORTB
     * RBIF presenza interrupt RB0
     */

	 
	// OPTION_REG
    OPTION = 0b10000000;
	/*
    bit 7 RBPU: PORTB Pull-up Enable bit
        1 = PORTB pull-ups are disabled
        0 = PORTB pull-ups are enabled by individual port latch values
    bit 6 INTEDG: Interrupt Edge Select bit
        1 = Interrupt on rising edge of RB0/INT pin
        0 = Interrupt on falling edge of RB0/INT pin
    bit 5 T0CS: TMR0 Clock Source Select bit
        1 = Transition on RA4/T0CKI pin
        0 = Internal instruction cycle clock (CLKO)
    bit 4 T0SE: TMR0 Source Edge Select bit
        1 = Increment on high-to-low transition on RA4/T0CKI pin
        0 = Increment on low-to-high transition on RA4/T0CKI pin
    bit 3 PSA: Prescaler Assignment bit
        1 = Prescaler is assigned to the WDT
        0 = Prescaler is assigned to the Timer0 module
    bit 2-0 PS2:PS0: Prescaler Rate Select bits
	*/


	 //Inizializzazione uart
    TXSTA = 0x20; // TX abilitato
    RCSTA = 0X90; // Registro RX USART
    BRGH = 0; // USART High velocity
    SPBRG = 15; // Asynchronous abbinato a BRGH = 0, Baud Rate Generator (15=19200 baud 20Mhz)

    //Inizializzazione TIMER1
    T1CON = 0; // TIMER 1 resettato
    T1CKPS1 = 0, T1CKPS0 = 0; // TIMER1 prescaler = 1
    T1OSCEN = 0; // TIMER1 external oscillator disable
    TMR1CS = 0; // TIMER1 internal clock (fosc/4)
    TMR1ON = 0; // TIMER1 in STOP

}